;; -*- mode: common-lisp -*-
;;
;; Copyright (C) 2008 Franz Inc, Oakland, CA.  All rights reserved.
;;
;; This code is free software; you can redistribute it and/or
;; modify it under the terms of the version 2.1 of
;; the GNU Lesser General Public License as published by 
;; the Free Software Foundation, as clarified by the Franz
;; preamble to the LGPL found in
;; http://opensource.franz.com/preamble.html.
;;
;; This code is distributed in the hope that it will be useful,
;; but without any warranty; without even the implied warranty of
;; merchantability or fitness for a particular purpose.  See the GNU
;; Lesser General Public License for more details.
;;
;; Version 2.1 of the GNU Lesser General Public License can be
;; found at http://opensource.franz.com/license.html.
;; If it is not present, you can access it from
;; http://www.gnu.org/copyleft/lesser.txt (until superseded by a newer
;; version) or write to the Free Software Foundation, Inc., 59 Temple
;; Place, Suite 330, Boston, MA  02111-1307  USA
;;
;; $Id$

(in-package :user)

(defvar *usage*
    "~
Usage: sample-server [-d] [-D] [-o logfile] port
  -d         -- debugging mode
  -D         -- daemon mode
  -o logfile -- logging to this file, or www.out if not given
  port       -- the port on which we listen
")

(defun main ()
  ;; Currently, using detach-from-terminal means this won't work on Windows.
  ;; However, the example should be modified to make the web server into a
  ;; service.
  (handler-case
      (sys:with-command-line-arguments ("Ddo:" daemonize debug logfile)
	  (rest)
	(when (/= 1 (length rest))
	  (format t *usage*)
	  (exit 0))
	
	(when (null logfile) (setq logfile (merge-pathnames "www.out" *logroot*)))

	(init-server (read-from-string (first rest)))
	
	(when debug
	  (setq *debug* t)
	  (break "debug away..."))
	#-mswindows
	(when daemonize
	  (format t ";; Logging to ~a...~%" logfile)
	  (with-open-file (logstream logfile :direction :output
			   :if-exists :always-append
			   :if-does-not-exist :create)
	    (if* (= (excl.osi:fork) 0)
	       then ;; child
		    (excl.osi:detach-from-terminal
		     :output-stream logstream
		     :error-output-stream logstream)
		    (format t ";; Web server started ~:@/locale-format-time/~%"
			    (get-universal-time))
		    (force-output)
	       else ;; parent
		    ;; exit fails if :no-unwind nil. bug16172.
		    (exit 0 :quiet t :no-unwind t))))
	
	(when (not debug) (loop (sleep most-positive-fixnum))))
    (error (c) (error-die "An error occurred: ~a." c))))

(defun init-server (port)
  (multiple-value-setq (*server* *reload-server*) (bind-server-logs '(:new :new)))
  (start :port (setq *port* port) :listeners 5 :server *server* :proxy t)
  (start :port (+ port 1000) :listeners 2 :server *reload-server*)
  
  (when (eql 80 port)
    (setf (wserver-filters *server*) '(filter-requests)))
  
  (mp:process-run-function "telnet server" 'start-telnet-server (+ port 1))
  
  ;; make sure telnet server process runs before we fork...
  (multiprocessing:process-allow-schedule)
  (reload)

  ;; For maximum debugging information at the web server level, do this:
  #+ignore (net.aserve::debug-on :all))
